<?php

namespace Myriad\Authentication\Http\Controllers;

use Illuminate\Routing\Controller;
use Illuminate\Contracts\Bus\Dispatcher;
use Myriad\Authentication\Lib\Jobs\RegisterUserWithEmailFromInstallation;
use Myriad\Installations\InstallationManager;
use Myriad\Authentication\Lib\Jobs\RegisterUser;
use Myriad\Authentication\Lib\Repositories\IAccessTokenRepository;

class RegistrationController extends Controller {

  private $dispatcher;

  public function __construct(Dispatcher $dispatcher) {
    $this->dispatcher = $dispatcher;
  }

  /**
   * Register a User
   *
   * @return void
   */
  public function register(
    \Illuminate\Http\Request $request, 
    InstallationManager $manager,
    IAccessTokenRepository $accessTokenRepository)
  {
    $user = $this->dispatcher->dispatchNow(new RegisterUser($request->all()));

    $installation = $manager->getInstallation();

    $accessToken = $accessTokenRepository->createAccessToken($user, $installation);

    return [
      'data' => [
        'access_token' => $accessToken->token,
        'expires_at' => $accessToken->expires_at->toIso8601String()
      ]
    ];
  }
}
