<?php

namespace Myriad\Authentication\Lib\Jobs;

use Illuminate\Contracts\Validation\ValidationException;
use Illuminate\Contracts\Validation\Factory;
use Myriad\Authentication\Lib\Repositories\IVerificationCodeRepository;
use Illuminate\Contracts\Auth\Guard;

class VerifyVerificationCode
{
  private $data;

  /**
   * @param array $data
   */
  public function __construct($data)
  {
    $this->data = $data;
  }

  public static function getRules() {
    return [
      'contact' => 'required|string',
      'code' => 'required|string'
    ];
  }

  public static function validate(Factory $validationFactory, array $data) {
    // validate request
    $validator = $validationFactory->make($data, self::getRules());

    if ($validator->fails()) {
      throw new ValidationException($validator);
    }
  }

  public function handle(
    Factory $factory,
    IVerificationCodeRepository $verificationCodeRepository,
    Guard $guard
  ) {
    self::validate($factory, $this->data);

    $codes = $verificationCodeRepository->findVerificationCodes($this->data['contact']);

    $matchingCode = $this->findMatch($codes);

    if ($matchingCode) {
      $user = $matchingCode->performVerificationStrategy($this->data['code']);
      $guard->setUser($user);
    }

    return null;
  }


  private function findMatch($codes) {
    foreach ($codes as $code) {
      if ($code->attemptToMatch($this->data['code'])) {
        return $code;
      }
    }
  }
 
}
