<?php

namespace Myriad\Authentication\Lib\Strategies;

use Myriad\Authentication\Lib\Services\IPhoneNumberConfirmer;
use Myriad\Authentication\Lib\Strategies\IVerificationStrategy;

class PhoneNumberVerification implements IVerificationStrategy, IPhoneNumberConfirmer
{
  use CreatesVerificationCodes;

  private $userRepository;
  private $mailer;
  private $config;

  /**
   * @param \Myriad\Authentication\Lib\Repositories\IUserRepository $userRepository
   */
  public function __construct(
    \Myriad\Authentication\Lib\Repositories\IUserRepository $userRepository,
    \Illuminate\Contracts\Mail\Mailer $mailer,
    \Illuminate\Contracts\Config\Repository $config
  ) {
    $this->userRepository = $userRepository;
    $this->mailer = $mailer;
    $this->config = $config;
  }

  public function sendConfirmationTextMessage($phoneNumber, $user, array $attributes) {
    //TODO: for now this is just sending an email to systems@myriadmobile.com...not actually
    //      doing its job yet. i think all current constructor dependencies can be removed them as well
    $attributes = [
      'type' => array_key_exists('type', $attributes) ? $attributes['type'] : 'personal',
      'primary' => array_key_exists('primary', $attributes) ? $attributes['primary'] : false,
      'user_id' => is_null($user) ? null : $user->getAuthIdentifier(),
    ];

    $code = $this->createVerificationCode( $phoneNumber, PhoneNumberVerification::class, $attributes);

    $emailTemplate = $this->config->get('authentication.email.templates.email_address_verification');

    // send an email asking user to verify their code
    $this->mailer->send(
      $emailTemplate['template'],
      [
        'code' => $code->code,
        'app_name' => $this->config->get('authentication.email.app_name')
      ],
      function($message) use ($emailTemplate) {
        $message->to('systems@myriadmobile.com')->subject($emailTemplate['subject']);
      }
    );
  }

  public function performVerificationStrategy(array $data) {
    $customData = json_decode($data['data'], true);
    $email = trim($data['contact']);

    //TODO: abstract this behind repository
    $existingUserEmail = UserEmail::where('email', $email)->where('verified', true)->first();

    // check if verification code was created when user was already logged in
    if (array_key_exists('user_id', $customData) && !is_null($customData['user_id'])) {
      $user = $this->userRepository->retrieveById($customData['user_id']);

      if ($existingUserEmail) {
        if ($user->id !== $existingUserEmail->user->id) {
          //TODO: throw a better exception
          throw new \Exception("Email address belongs to a different user!");
        }
      } else {
        // no existing user attached to email, so we're ok to create/update email for our user
        $userEmail = $user->emailAddresses()->where('email', $email)->first();

        if ($userEmail) {
          $userEmail->verified = true;
          $userEmail->save();
        } else {
          $user->emailAddresses()->create(array_merge(
            [ 'email' => $email ],
            $customData,
            [ 'verified' => true ]
          ));
        }
      }
    } else {
      if ($existingUserEmail) {
        //TODO: throw a better exception
        throw new \Exception("Email address belongs to a different user!");
      } else {
        // no existing user attached to email, so we're ok to create a user and attach email
        $user = $this->userRepository->createUser();
        
        $user->emailAddresses()->create(array_merge(
          [ 'email' => $email ],
          $customData,
          [ 'verified' => true ]
        ));
      }
    }

    return $user;
  }
}
